@@ -2,12 +2,12 @@ import os
 
 from starlette.responses import JSONResponse, PlainTextResponse
 
-from langgraph_api import config
+from langgraph_api import config, metadata
 from langgraph_api.route import ApiRequest
 from langgraph_license.validation import plus_features_enabled
-from langgraph_storage.database import connect, pool_stats
-from langgraph_storage.ops import Runs
-from langgraph_storage.queue import WORKERS
+from langgraph_runtime.database import connect, pool_stats
+from langgraph_runtime.metrics import get_metrics
+from langgraph_runtime.ops import Runs
 
 METRICS_FORMATS = {"prometheus", "json"}
 
@@ -20,7 +20,13 @@ async def meta_info(request: ApiRequest):
                 "assistants": True,
                 "crons": plus and config.FF_CRONS_ENABLED,
                 "langsmith": bool(config.LANGSMITH_API_KEY) and bool(config.TRACING),
-            }
+            },
+            "host": {
+                "kind": metadata.HOST,
+                "project_id": metadata.PROJECT_ID,
+                "revision_id": metadata.REVISION,
+                "tenant_id": metadata.TENANT_ID,
+            },
         }
     )
 
@@ -32,20 +38,18 @@ async def meta_metrics(request: ApiRequest):
         format = "prometheus"
 
     # collect stats
-    workers_max = config.N_JOBS_PER_WORKER
-    workers_active = len(WORKERS)
-    workers_available = workers_max - workers_active
+    metrics = get_metrics()
+    worker_metrics = metrics["workers"]
+    workers_max = worker_metrics["max"]
+    workers_active = worker_metrics["active"]
+    workers_available = worker_metrics["available"]
 
     if format == "json":
         async with connect() as conn:
             return JSONResponse(
                 {
                     **pool_stats(),
-                    "workers": {
-                        "max": workers_max,
-                        "active": workers_active,
-                        "available": workers_available,
-                    },
+                    "workers": worker_metrics,
                     "queue": await Runs.stats(conn),
                 }
             )
