@@ -1,8 +1,10 @@
 import asyncio
 from collections.abc import AsyncIterator
+from typing import Literal
 
 import orjson
 from langgraph.checkpoint.base.id import uuid6
+from starlette.exceptions import HTTPException
 from starlette.responses import Response, StreamingResponse
 
 from langgraph_api import config
@@ -17,11 +19,12 @@ from langgraph_api.validation import (
     RunBatchCreate,
     RunCreateStateful,
     RunCreateStateless,
+    RunsCancel,
 )
 from langgraph_license.validation import plus_features_enabled
-from langgraph_storage.database import connect
-from langgraph_storage.ops import Crons, Runs, Threads
-from langgraph_storage.retry import retry_db
+from langgraph_runtime.database import connect
+from langgraph_runtime.ops import Crons, Runs, Threads
+from langgraph_runtime.retry import retry_db
 
 
 @retry_db
@@ -270,7 +273,7 @@ async def wait_run_stateless(request: ApiRequest):
             except TimeoutError:
                 yield b"\n"
             except asyncio.CancelledError:
-                stream.cancel()
+                stream.cancel("Run stream cancelled")
                 await stream
                 raise
 
@@ -354,11 +357,13 @@ async def join_run_stream_endpoint(request: ApiRequest):
     cancel_on_disconnect = cancel_on_disconnect_str.lower() in {"true", "yes", "1"}
     validate_uuid(thread_id, "Invalid thread ID: must be a UUID")
     validate_uuid(run_id, "Invalid run ID: must be a UUID")
+    stream_mode = request.query_params.get("stream_mode") or None
     return EventSourceResponse(
         Runs.Stream.join(
             run_id,
             thread_id=thread_id,
             cancel_on_disconnect=cancel_on_disconnect,
+            stream_mode=stream_mode,
         )
     )
 
@@ -392,6 +397,44 @@ async def cancel_run(
     return Response(status_code=204 if wait else 202)
 
 
+@retry_db
+async def cancel_runs(
+    request: ApiRequest,
+):
+    """Cancel a run."""
+    body = await request.json(RunsCancel)
+    status = body.get("status")
+    if status:
+        status = status.lower()
+        if status not in ("pending", "running", "all"):
+            raise HTTPException(
+                status_code=422,
+                detail="Invalid status: must be 'pending', 'running', or 'all'",
+            )
+        thread_id = None
+        run_ids = None
+    else:
+        thread_id = body.get("thread_id")
+        run_ids = body.get("run_ids")
+        validate_uuid(thread_id, "Invalid thread ID: must be a UUID")
+        for rid in run_ids:
+            validate_uuid(rid, "Invalid run ID: must be a UUID")
+    action_str = request.query_params.get("action", "interrupt")
+    action: Literal["interrupt", "rollback"] = (
+        action_str if action_str in ("interrupt", "rollback") else "interrupt"
+    )
+
+    async with connect() as conn:
+        await Runs.cancel(
+            conn,
+            run_ids,
+            action=action,
+            thread_id=thread_id,
+            status=status,
+        )
+    return Response(status_code=204)
+
+
 @retry_db
 async def delete_run(request: ApiRequest):
     """Delete a run by ID."""
@@ -484,6 +527,7 @@ runs_routes = [
     ApiRoute("/runs/wait", wait_run_stateless, methods=["POST"]),
     ApiRoute("/runs", create_stateless_run, methods=["POST"]),
     ApiRoute("/runs/batch", create_stateless_run_batch, methods=["POST"]),
+    ApiRoute("/runs/cancel", cancel_runs, methods=["POST"]),
     (
         ApiRoute("/runs/crons", create_cron, methods=["POST"])
         if config.FF_CRONS_ENABLED and plus_features_enabled()
