@@ -1,4 +1,5 @@
 import asyncio
+import concurrent.futures
 from collections.abc import AsyncIterator, Coroutine
 from contextlib import AbstractAsyncContextManager
 from functools import partial
@@ -10,6 +11,13 @@ T = TypeVar("T")
 
 logger = structlog.stdlib.get_logger(__name__)
 
+_MAIN_LOOP: asyncio.AbstractEventLoop | None = None
+
+
+def set_event_loop(loop: asyncio.AbstractEventLoop) -> None:
+    global _MAIN_LOOP
+    _MAIN_LOOP = loop
+
 
 async def sleep_if_not_done(delay: float, done: asyncio.Event) -> None:
     try:
@@ -55,7 +63,9 @@ async def wait_if_not_done(coro: Coroutine[Any, Any, T], done: ValueEvent) -> T:
         async with asyncio.TaskGroup() as tg:
             coro_task = tg.create_task(coro)
             done_task = tg.create_task(done.wait())
-            coro_task.add_done_callback(lambda _: done_task.cancel())
+            coro_task.add_done_callback(
+                lambda _: done_task.cancel("Coro task completed")
+            )
             done_task.add_done_callback(lambda _: coro_task.cancel(done._value))
             try:
                 return await coro_task
@@ -76,9 +86,10 @@ PENDING_TASKS = set()
 
 
 def _create_task_done_callback(
-    ignore_exceptions: tuple[Exception, ...], task: asyncio.Task
+    ignore_exceptions: tuple[Exception, ...],
+    task: asyncio.Task | asyncio.Future,
 ) -> None:
-    PENDING_TASKS.remove(task)
+    PENDING_TASKS.discard(task)
     try:
         if exc := task.exception():
             if not isinstance(exc, ignore_exceptions):
@@ -97,6 +108,16 @@ def create_task(
     return task
 
 
+def run_coroutine_threadsafe(
+    coro: Coroutine[Any, Any, T], ignore_exceptions: tuple[type[Exception], ...] = ()
+) -> concurrent.futures.Future[T | None]:
+    if _MAIN_LOOP is None:
+        raise RuntimeError("No event loop set")
+    future = asyncio.run_coroutine_threadsafe(coro, _MAIN_LOOP)
+    future.add_done_callback(partial(_create_task_done_callback, ignore_exceptions))
+    return future
+
+
 class SimpleTaskGroup(AbstractAsyncContextManager["SimpleTaskGroup"]):
     """An async task group that can be configured to wait and/or cancel tasks on exit.
 
@@ -112,6 +133,7 @@ class SimpleTaskGroup(AbstractAsyncContextManager["SimpleTaskGroup"]):
         cancel: bool = False,
         wait: bool = True,
         taskset: set[asyncio.Task] | None = None,
+        taskgroup_name: str | None = None,
     ) -> None:
         self.tasks = taskset if taskset is not None else set()
         self.cancel = cancel
@@ -121,6 +143,7 @@ class SimpleTaskGroup(AbstractAsyncContextManager["SimpleTaskGroup"]):
                 task.add_done_callback(partial(self._create_task_done_callback, ()))
         for coro in coros:
             self.create_task(coro)
+        self.taskgroup_name = f" {taskgroup_name} " if taskgroup_name else ""
 
     def _create_task_done_callback(
         self, ignore_exceptions: tuple[Exception, ...], task: asyncio.Task
@@ -156,7 +179,7 @@ class SimpleTaskGroup(AbstractAsyncContextManager["SimpleTaskGroup"]):
         # cancel all tasks
         if self.cancel:
             for task in tasks:
-                task.cancel()
+                task.cancel(f"Task group{self.taskgroup_name}cancelled.")
         # wait for all tasks
         if self.wait:
             await asyncio.gather(*tasks, return_exceptions=True)
