@@ -7,7 +7,7 @@ import os
 import sys
 from collections.abc import Awaitable, Callable, Mapping
 from contextlib import AsyncExitStack
-from typing import Any, get_args
+from typing import Any, Literal, get_args
 
 import structlog
 from langgraph_sdk import Auth
@@ -18,6 +18,7 @@ from starlette.authentication import (
     BaseUser,
 )
 from starlette.concurrency import run_in_threadpool
+from starlette.datastructures import QueryParams
 from starlette.exceptions import HTTPException
 from starlette.requests import HTTPConnection, Request
 from starlette.responses import Response
@@ -25,6 +26,7 @@ from starlette.responses import Response
 from langgraph_api.auth.langsmith.backend import LangsmithAuthBackend
 from langgraph_api.auth.studio_user import StudioUser
 from langgraph_api.config import LANGGRAPH_AUTH, LANGGRAPH_AUTH_TYPE
+from langgraph_api.js.base import is_js_path
 
 logger = structlog.stdlib.get_logger(__name__)
 
@@ -37,7 +39,7 @@ SUPPORTED_PARAMETERS = {
     "scopes": list[str],
     "path_params": dict[str, str] | None,
     "query_params": dict[str, str] | None,
-    "headers": dict[str, bytes] | None,
+    "headers": dict[bytes, bytes] | None,
     "authorization": str | None,
     "scope": dict[str, Any],
 }
@@ -55,7 +57,7 @@ def get_custom_auth_middleware() -> AuthenticationBackend:
 
 
 @functools.lru_cache(maxsize=1)
-def get_auth_instance() -> Auth | None:
+def get_auth_instance() -> Auth | Literal["js"] | None:
     logger.info(
         f"Getting auth instance: {LANGGRAPH_AUTH}", langgraph_auth=str(LANGGRAPH_AUTH)
     )
@@ -88,6 +90,12 @@ async def handle_event(
     auth = get_auth_instance()
     if auth is None:
         return
+
+    if auth == "js":
+        from langgraph_api.js.remote import handle_js_auth_event
+
+        return await handle_js_auth_event(ctx, value)
+
     handler = _get_handler(auth, ctx)
     if not handler:
         return
@@ -175,21 +183,14 @@ class CustomAuthBackend(AuthenticationBackend):
         except (AuthenticationError, HTTPException):
             raise
         except Auth.exceptions.HTTPException as e:
-            raise HTTPException(
-                status_code=e.status_code,
-                detail=e.detail,
-                headers=dict(e.headers) if e.headers else None,
-            ) from None
-        except AssertionError as e:
-            raise AuthenticationError(str(e)) from None
+            if e.status_code == 401 or e.status_code == 403:
+                raise AuthenticationError(e.detail) from None
+            else:
+                await logger.aerror("Error authenticating request", exc_info=e)
+                raise
         except Exception as e:
             await logger.aerror("Error authenticating request", exc_info=e)
-            status_code = getattr(e, "status_code", 401)
-            detail = getattr(e, "detail", "Unauthorized")
-            headers = getattr(e, "headers", None)
-            raise HTTPException(
-                status_code=status_code, detail=detail, headers=headers
-            ) from None
+            raise e
 
 
 def _get_custom_auth_middleware(
@@ -201,6 +202,7 @@ def _get_custom_auth_middleware(
     else:
         path = config.get("path")
         disable_studio_auth = config.get("disable_studio_auth", disable_studio_auth)
+
     auth_instance = _get_auth_instance(path)
     if auth_instance is None:
         raise ValueError(
@@ -210,6 +212,12 @@ def _get_custom_auth_middleware(
             "from langgraph_sdk import Auth\n"
             "auth = Auth()"
         )
+
+    if auth_instance == "js":
+        from langgraph_api.js.remote import CustomJsAuthBackend
+
+        return CustomJsAuthBackend(disable_studio_auth=disable_studio_auth)
+
     if auth_instance._authenticate_handler is None:
         raise ValueError(
             f"Custom Auth object at path: {path} does not have an authenticate handler."
@@ -220,6 +228,7 @@ def _get_custom_auth_middleware(
             "async def authenticate(request):\n"
             '    return "my-user-id"'
         )
+
     result = CustomAuthBackend(
         auth_instance._authenticate_handler,
         disable_studio_auth,
@@ -229,12 +238,15 @@ def _get_custom_auth_middleware(
 
 
 @functools.lru_cache(maxsize=1)
-def _get_auth_instance(path: str | None = None) -> Auth | None:
+def _get_auth_instance(path: str | None = None) -> Auth | Literal["js"] | None:
     if path is not None:
         auth_instance = _load_auth_obj(path)
     else:
         auth_instance = None
 
+    if auth_instance == "js":
+        return auth_instance
+
     if auth_instance is not None and (
         deps := _get_dependencies(auth_instance._authenticate_handler)
     ):
@@ -271,7 +283,7 @@ def _extract_arguments_from_scope(
     if "path" in param_names:
         args["path"] = scope["path"]
     if "query_params" in param_names:
-        args["query_params"] = scope.get("query_params", {})
+        args["query_params"] = QueryParams(scope.get("query_string"))
     if "headers" in param_names:
         args["headers"] = dict(scope.get("headers", {}))
     if "authorization" in param_names:
@@ -385,13 +397,16 @@ def _depends() -> Any:
         return None
 
 
+_EXCLUDED = ("values", "keys", "items", "dict")
+
+
 class DotDict:
     def __init__(self, dictionary: dict[str, Any]):
         self._dict = dictionary
         for key, value in dictionary.items():
             if isinstance(value, dict):
                 setattr(self, key, DotDict(value))
-            else:
+            elif key not in _EXCLUDED:
                 setattr(self, key, value)
 
     def __getattr__(self, name):
@@ -399,6 +414,9 @@ class DotDict:
             raise AttributeError(f"'DotDict' object has no attribute '{name}'")
         return self._dict[name]
 
+    def __contains__(self, key: str) -> bool:
+        return key in self._dict
+
     def __getitem__(self, key):
         return self._dict[key]
 
@@ -415,6 +433,21 @@ class DotDict:
     def dict(self):
         return self._dict
 
+    def items(self):
+        return self._dict.items()
+
+    def values(self):
+        return self._dict.values()
+
+    def keys(self):
+        return self._dict.keys()
+
+    def __iter__(self):
+        return iter(self._dict)
+
+    def __len__(self):
+        return len(self._dict)
+
 
 class ProxyUser(BaseUser):
     """A proxy that wraps a user object to ensure it has all BaseUser properties.
@@ -468,6 +501,9 @@ class ProxyUser(BaseUser):
             **d,
         }
 
+    def __contains__(self, key: str) -> bool:
+        return key in self._user
+
     def __getitem__(self, key):
         return self._user[key]
 
@@ -478,6 +514,9 @@ class ProxyUser(BaseUser):
         """Proxy any other attributes to the underlying user object."""
         return getattr(self._user, name)
 
+    def __str__(self) -> str:
+        return f"{self._user}"
+
 
 class SimpleUser(ProxyUser):
     def __init__(self, username: str):
@@ -523,7 +562,7 @@ def normalize_user(user: Any) -> BaseUser:
     )
 
 
-def _load_auth_obj(path: str) -> Auth:
+def _load_auth_obj(path: str) -> Auth | Literal["js"]:
     """Load an object from a path string."""
     if ":" not in path:
         raise ValueError(
@@ -534,6 +573,9 @@ def _load_auth_obj(path: str) -> Auth:
     module_name, callable_name = path.rsplit(":", 1)
     module_name = module_name.rstrip(":")
 
+    if is_js_path(module_name):
+        return "js"
+
     try:
         if "/" in module_name or ".py" in module_name:
             # Load from file path
