@@ -6,6 +6,7 @@ import inspect
 import json
 import os
 import sys
+import warnings
 from collections.abc import AsyncIterator, Callable
 from contextlib import asynccontextmanager
 from itertools import filterfalse
@@ -21,7 +22,9 @@ from langgraph.pregel import Pregel
 from langgraph.store.base import BaseStore
 from starlette.exceptions import HTTPException
 
-from langgraph_api.js.base import BaseRemotePregel
+from langgraph_api import asyncio as lg_asyncio
+from langgraph_api import config
+from langgraph_api.js.base import BaseRemotePregel, is_js_path
 from langgraph_api.schema import Config
 
 if TYPE_CHECKING:
@@ -39,10 +42,16 @@ NAMESPACE_GRAPH = UUID("6ba7b821-9dad-11d1-80b4-00c04fd430c8")
 FACTORY_ACCEPTS_CONFIG: dict[str, bool] = {}
 
 
-async def register_graph(graph_id: str, graph: GraphValue, config: dict | None) -> None:
+async def register_graph(
+    graph_id: str,
+    graph: GraphValue,
+    config: dict | None,
+    *,
+    description: str | None = None,
+) -> None:
     """Register a graph."""
-    from langgraph_storage.database import connect
-    from langgraph_storage.ops import Assistants
+    from langgraph_runtime.database import connect
+    from langgraph_runtime.ops import Assistants
 
     await logger.ainfo(f"Registering graph with id '{graph_id}'", graph_id=graph_id)
     GRAPHS[graph_id] = graph
@@ -57,9 +66,16 @@ async def register_graph(graph_id: str, graph: GraphValue, config: dict | None)
             config=config or {},
             if_exists="do_nothing",
             name=graph_id,
+            description=description,
         )
 
 
+def register_graph_sync(
+    graph_id: str, graph: GraphValue, config: dict | None = None
+) -> None:
+    lg_asyncio.run_coroutine_threadsafe(register_graph(graph_id, graph, config))
+
+
 @asynccontextmanager
 async def _generate_graph(value: Any) -> AsyncIterator[Any]:
     """Yield a graph object regardless of its type."""
@@ -104,18 +120,14 @@ async def get_graph(
                 status_code=424,
                 detail=f"Graph '{graph_id}' is not valid. Review graph registration.",
             )
-        if isinstance(graph_obj, BaseRemotePregel):
-            graph_obj.checkpointer = checkpointer
-            graph_obj.name = graph_id
-            yield graph_obj
-            return
-
         update = {
             "checkpointer": checkpointer,
             "store": store,
         }
         if graph_obj.name == "LangGraph":
             update["name"] = graph_id
+        if isinstance(graph_obj, BaseRemotePregel):
+            update["config"] = config
         yield graph_obj.copy(update=update)
 
 
@@ -144,29 +156,31 @@ def get_assistant_id(assistant_id: str) -> str:
 
 
 class GraphSpec(NamedTuple):
-    """A graph specification."""
+    """A graph specification.
+
+    This is a definition of the graph that can be used to load the graph
+    from a file or module.
+    """
 
     id: str
+    """The ID of the graph."""
     path: str | None = None
     module: str | None = None
     variable: str | None = None
     config: dict | None = None
+    """The configuration for the graph.
+    
+    Contains information such as: tags, recursion_limit and configurable.
+    
+    Configurable is a dict containing user defined values for the graph.
+    """
+    description: str | None = None
+    """A description of the graph"""
 
 
 js_bg_tasks: set[asyncio.Task] = set()
 
 
-def is_js_spec(spec: GraphSpec) -> bool:
-    return spec.path is not None and os.path.splitext(spec.path)[1] in (
-        ".ts",
-        ".mts",
-        ".cts",
-        ".js",
-        ".mjs",
-        ".cjs",
-    )
-
-
 def _load_graph_config_from_env() -> dict | None:
     """Return graph config from env."""
     config_str = os.getenv("LANGGRAPH_CONFIG")
@@ -187,24 +201,65 @@ async def collect_graphs_from_env(register: bool = False) -> None:
     config_per_graph = _load_graph_config_from_env() or {}
 
     if paths_str:
-        specs = [
-            (
+        specs = []
+        # graphs-config can be either a mapping from graph id to path where the graph
+        # is defined or graph id to a dictionary containing information about the graph.
+        graphs_config = json.loads(paths_str)
+
+        for key, value in graphs_config.items():
+            if isinstance(value, dict) and "path" in value:
+                source = value["path"]
+            elif isinstance(value, str):
+                source = value
+            else:
+                msg = (
+                    f"Invalid value '{value}' for graph '{key}'. "
+                    "Expected a string or a dictionary. "
+                    "If a string, it should be the path to the graph definition. "
+                    "For example: '/path/to/graph.py:graph_variable' "
+                    "or 'my.module:graph_variable'. "
+                    "If a dictionary, then it needs to contains a `path` key with the "
+                    "path to the graph definition."
+                    "It can also contains additional configuration for the graph; "
+                    "e.g., `description`."
+                    "For example: {'path': '/path/to/graph.py:graph_variable', "
+                    "'description': 'My graph'}"
+                )
+                raise TypeError(msg)
+
+            try:
+                path_or_module, variable = source.rsplit(":", maxsplit=1)
+            except ValueError as e:
+                raise ValueError(
+                    f"Invalid path '{value}' for graph '{key}'."
+                    " Did you miss a variable name?\n"
+                    " Expected one of the following formats:"
+                    " 'my.module:variable_name' or '/path/to/file.py:variable_name'"
+                ) from e
+
+            graph_config = config_per_graph.get(key, {})
+            description = (
+                value.get("description", None) if isinstance(value, dict) else None
+            )
+
+            # Module syntax uses `.` instead of `/` to separate directories
+            if "/" in path_or_module:
+                path = path_or_module
+                module_ = None
+            else:
+                path = None
+                module_ = path_or_module
+
+            specs.append(
                 GraphSpec(
                     key,
-                    module=value.split(":")[0],
-                    variable=value.split(":")[1],
-                    config=config_per_graph.get(key),
-                )
-                if "/" not in value
-                else GraphSpec(
-                    key,
-                    path=value.split(":")[0],
-                    variable=value.split(":")[1],
-                    config=config_per_graph.get(key),
+                    module=module_,
+                    path=path,
+                    variable=variable,
+                    config=graph_config,
+                    description=description,
                 )
             )
-            for key, value in json.loads(paths_str).items()
-        ]
     else:
         specs = [
             GraphSpec(
@@ -217,11 +272,14 @@ async def collect_graphs_from_env(register: bool = False) -> None:
             for graph_path in glob.glob("/graphs/*.py")
         ]
 
+    def is_js_spec(x: GraphSpec) -> bool:
+        return is_js_path(x.path)
+
     js_specs = list(filter(is_js_spec, specs))
     py_specs = list(filterfalse(is_js_spec, specs))
 
     if js_specs:
-        if os.environ.get("LANGSMITH_LANGGRAPH_API_VARIANT") == "local_dev":
+        if config.API_VARIANT == "local_dev":
             raise NotImplementedError(
                 "LangGraph.JS graphs are not yet supported in local development mode. "
                 "To run your JS graphs, either use the LangGraph Studio application "
@@ -231,6 +289,7 @@ async def collect_graphs_from_env(register: bool = False) -> None:
 
         from langgraph_api.js.remote import (
             RemotePregel,
+            run_js_http_process,
             run_js_process,
             run_remote_checkpointer,
             wait_until_js_ready,
@@ -248,20 +307,40 @@ async def collect_graphs_from_env(register: bool = False) -> None:
                 name="remote-graphs",
             )
         )
+
+        if (
+            config.HTTP_CONFIG
+            and config.HTTP_CONFIG.get("app")
+            and is_js_path(config.HTTP_CONFIG.get("app").split(":")[0])
+        ):
+            js_bg_tasks.add(
+                asyncio.create_task(
+                    run_js_http_process(
+                        paths_str,
+                        config.HTTP_CONFIG.get("app"),
+                        watch="--reload" in sys.argv[1:],
+                    ),
+                )
+            )
+
         for task in js_bg_tasks:
             task.add_done_callback(_handle_exception)
 
         await wait_until_js_ready()
 
         for spec in js_specs:
-            graph = RemotePregel.load(graph_id=spec.id)
+            graph = RemotePregel(graph_id=spec.id)
             if register:
-                await register_graph(spec.id, graph, spec.config)
+                await register_graph(
+                    spec.id, graph, spec.config, description=spec.description
+                )
 
     for spec in py_specs:
         graph = await run_in_executor(None, _graph_from_spec, spec)
         if register:
-            await register_graph(spec.id, graph, spec.config)
+            await register_graph(
+                spec.id, graph, spec.config, description=spec.description
+            )
 
 
 def _handle_exception(task: asyncio.Task) -> None:
@@ -275,9 +354,9 @@ def _handle_exception(task: asyncio.Task) -> None:
 
 
 async def stop_remote_graphs() -> None:
-    logger.info("Cancelling remote graphs")
+    logger.info("Shutting down remote graphs")
     for task in js_bg_tasks:
-        task.cancel()
+        task.cancel("Stopping remote graphs.")
 
 
 def verify_graphs() -> None:
@@ -300,7 +379,7 @@ def _graph_from_spec(spec: GraphSpec) -> GraphValue:
             modspec.loader.exec_module(module)
         except ImportError as e:
             e.add_note(f"Could not import python module for graph:\n{spec}")
-            if os.environ.get("LANGSMITH_LANGGRAPH_API_VARIANT") == "local_dev":
+            if config.API_VARIANT == "local_dev":
                 e.add_note(
                     "This error likely means you haven't installed your project and its dependencies yet. Before running the server, install your project:\n\n"
                     "If you are using requirements.txt:\n"
@@ -358,7 +437,31 @@ def _graph_from_spec(spec: GraphSpec) -> GraphValue:
         elif isinstance(graph, Graph):
             graph = graph.compile()
         elif isinstance(graph, Pregel):
-            pass
+            # We don't want to fail real deployments, but this will help folks catch unnecessary custom components
+            # before they deploy
+            if config.API_VARIANT == "local_dev":
+                has_checkpointer = isinstance(graph.checkpointer, BaseCheckpointSaver)
+                has_store = isinstance(graph.store, BaseStore)
+                if has_checkpointer or has_store:
+                    components = []
+                    if has_checkpointer:
+                        components.append(
+                            f"checkpointer (type {type(graph.checkpointer)})"
+                        )
+                    if has_store:
+                        components.append(f"store (type {type(graph.store)})")
+                    component_list = " and ".join(components)
+
+                    raise ValueError(
+                        f"Heads up! Your graph '{spec.variable}' from '{spec.path}' includes a custom {component_list}. "
+                        f"With LangGraph API, persistence is handled automatically by the platform, "
+                        f"so providing a custom {component_list} here isn't necessary and will be ignored when deployed.\n\n"
+                        f"To simplify your setup and use the built-in persistence, please remove the custom {component_list} "
+                        f"from your graph definition. If you are looking to customize which postgres database to connect to,"
+                        " please set the `POSTGRES_URI` environment variable."
+                        " See https://langchain-ai.github.io/langgraph/cloud/reference/env_var/#postgres_uri_custom for more details."
+                    )
+
         else:
             raise ValueError(
                 f"Variable '{spec.variable}' in module '{spec.path}' is not a Graph or Graph factory function"
@@ -450,7 +553,7 @@ def resolve_embeddings(index_config: dict) -> "Embeddings":
 
         except ImportError as e:
             e.add_note(f"Could not import embeddings module:\n{module_name}\n\n")
-            if os.environ.get("LANGSMITH_LANGGRAPH_API_VARIANT") == "local_dev":
+            if config.API_VARIANT == "local_dev":
                 e.add_note(
                     "If you're in development mode, make sure you've installed your project "
                     "and its dependencies:\n"
@@ -472,4 +575,10 @@ def resolve_embeddings(index_config: dict) -> "Embeddings":
                 " or specify 'embed' as a path to a "
                 "variable in a Python file instead."
             )
-        return init_embeddings(embed)
+        # Capture warnings
+        with warnings.catch_warnings():
+            warnings.filterwarnings(
+                "ignore",
+                message=("The function `init_embeddings` is in beta."),
+            )
+            return init_embeddings(embed)
