@@ -1,9 +1,12 @@
 import asyncio
+import logging
 import os
 import shutil
 import ssl
-from collections.abc import AsyncIterator
-from typing import Any, Literal
+from collections import deque
+from collections.abc import AsyncIterator, Callable
+from contextlib import AbstractContextManager
+from typing import Any, Literal, Self, cast
 
 import certifi
 import httpx
@@ -21,24 +24,36 @@ from langchain_core.runnables.schema import (
 from langgraph.checkpoint.serde.base import SerializerProtocol
 from langgraph.pregel.types import PregelTask, StateSnapshot
 from langgraph.store.base import GetOp, Item, ListNamespacesOp, PutOp, SearchOp
-from langgraph.types import Command, Interrupt
+from langgraph.types import Command, Interrupt, Send
+from langgraph_sdk import Auth
 from pydantic import BaseModel
+from starlette import types
 from starlette.applications import Starlette
+from starlette.authentication import (
+    AuthCredentials,
+    AuthenticationBackend,
+    BaseUser,
+)
+from starlette.datastructures import MutableHeaders
 from starlette.exceptions import HTTPException
-from starlette.requests import Request
+from starlette.requests import HTTPConnection, Request
 from starlette.routing import Route
 
+from langgraph_api.auth.custom import DotDict, ProxyUser
+from langgraph_api.config import LANGGRAPH_AUTH_TYPE
 from langgraph_api.js.base import BaseRemotePregel
 from langgraph_api.js.errors import RemoteException
 from langgraph_api.js.sse import SSEDecoder, aiter_lines_raw
 from langgraph_api.route import ApiResponse
+from langgraph_api.schema import Config
 from langgraph_api.serde import json_dumpb
 from langgraph_api.utils import AsyncConnectionProto
 
 logger = structlog.stdlib.get_logger(__name__)
 
-GRAPH_PORT = 5556
 REMOTE_PORT = 5555
+GRAPH_PORT = 5556
+GRAPH_HTTP_PORT = 5557
 SSL = ssl.create_default_context(cafile=certifi.where())
 
 if port := int(os.getenv("PORT", "8080")):
@@ -55,6 +70,12 @@ _client = httpx.AsyncClient(
 )
 
 
+def default_command(obj):
+    if isinstance(obj, Send):
+        return {"node": obj.node, "args": obj.arg}
+    raise TypeError
+
+
 async def _client_stream(method: str, data: dict[str, Any]):
     graph_id = data.get("graph_id")
     async with _client.stream(
@@ -65,7 +86,7 @@ async def _client_stream(method: str, data: dict[str, Any]):
             "Cache-Control": "no-store",
             "Content-Type": "application/json",
         },
-        data=orjson.dumps(data),
+        data=orjson.dumps(data, default=default_command),
     ) as response:
         decoder = SSEDecoder()
         async for line in aiter_lines_raw(response):
@@ -81,17 +102,22 @@ async def _client_invoke(method: str, data: dict[str, Any]):
     res = await _client.post(
         f"/{graph_id}/{method}",
         headers={"Content-Type": "application/json"},
-        data=orjson.dumps(data),
+        data=orjson.dumps(data, default=default_command),
     )
     return res.json()
 
 
 class RemotePregel(BaseRemotePregel):
-    @staticmethod
-    def load(graph_id: str):
-        model = RemotePregel()
-        model.graph_id = graph_id
-        return model
+    def __init__(
+        self,
+        graph_id: str,
+        *,
+        config: Config | None = None,
+        **kwargs: Any,
+    ):
+        super().__init__()
+        self.graph_id = graph_id
+        self.config = config
 
     async def astream_events(
         self,
@@ -106,6 +132,8 @@ class RemotePregel(BaseRemotePregel):
 
         data = {
             "graph_id": self.graph_id,
+            "graph_config": self.config,
+            "graph_name": self.name,
             "command" if isinstance(input, Command) else "input": input,
             "config": config,
             **kwargs,
@@ -127,7 +155,14 @@ class RemotePregel(BaseRemotePregel):
         xray: int | bool = False,
     ) -> DrawableGraph:
         response = await _client_invoke(
-            "getGraph", {"graph_id": self.graph_id, "config": config, "xray": xray}
+            "getGraph",
+            {
+                "graph_id": self.graph_id,
+                "graph_config": self.config,
+                "graph_name": self.name,
+                "config": config,
+                "xray": xray,
+            },
         )
 
         nodes: list[Any] = response.pop("nodes")
@@ -155,11 +190,22 @@ class RemotePregel(BaseRemotePregel):
         )
 
     async def fetch_subgraphs(
-        self, *, namespace: str | None = None, recurse: bool = False
+        self,
+        *,
+        namespace: str | None = None,
+        config: RunnableConfig | None = None,
+        recurse: bool = False,
     ) -> dict[str, dict]:
         return await _client_invoke(
             "getSubgraphs",
-            {"graph_id": self.graph_id, "namespace": namespace, "recurse": recurse},
+            {
+                "graph_id": self.graph_id,
+                "graph_config": self.config,
+                "graph_name": self.name,
+                "namespace": namespace,
+                "recurse": recurse,
+                "config": config,
+            },
         )
 
     def _convert_state_snapshot(self, item: dict) -> StateSnapshot:
@@ -212,7 +258,13 @@ class RemotePregel(BaseRemotePregel):
         return self._convert_state_snapshot(
             await _client_invoke(
                 "getState",
-                {"graph_id": self.graph_id, "config": config, "subgraphs": subgraphs},
+                {
+                    "graph_id": self.graph_id,
+                    "graph_config": self.config,
+                    "graph_name": self.name,
+                    "config": config,
+                    "subgraphs": subgraphs,
+                },
             )
         )
 
@@ -226,6 +278,8 @@ class RemotePregel(BaseRemotePregel):
             "updateState",
             {
                 "graph_id": self.graph_id,
+                "graph_config": self.config,
+                "graph_name": self.name,
                 "config": config,
                 "values": values,
                 "as_node": as_node,
@@ -245,6 +299,8 @@ class RemotePregel(BaseRemotePregel):
             "getStateHistory",
             {
                 "graph_id": self.graph_id,
+                "graph_config": self.config,
+                "graph_name": self.name,
                 "config": config,
                 "limit": limit,
                 "filter": filter,
@@ -275,12 +331,18 @@ class RemotePregel(BaseRemotePregel):
     async def invoke(self, input: Any, config: RunnableConfig | None = None):
         raise Exception("Not implemented")
 
+    def copy(self, update: dict[str, Any] | None = None) -> Self:
+        attrs = {**self.__dict__, **(update or {})}
+        return self.__class__(**attrs)
+
 
 async def run_js_process(paths_str: str, watch: bool = False):
     # check if tsx is available
     tsx_path = shutil.which("tsx")
     if tsx_path is None:
-        raise FileNotFoundError("tsx not found in PATH")
+        raise FileNotFoundError(
+            "tsx not found in PATH. Please upgrade to latest LangGraph CLI to support running JS graphs."
+        )
     attempt = 0
     while not asyncio.current_task().cancelled():
         client_file = os.path.join(os.path.dirname(__file__), "client.mts")
@@ -316,8 +378,56 @@ async def run_js_process(paths_str: str, watch: bool = False):
                 attempt += 1
 
 
+async def run_js_http_process(paths_str: str, http_config: dict, watch: bool = False):
+    # check if tsx is available
+    tsx_path = shutil.which("tsx")
+    if tsx_path is None:
+        raise FileNotFoundError(
+            "tsx not found in PATH. Please upgrade to latest LangGraph CLI to support running JS graphs."
+        )
+
+    attempt = 0
+    while not asyncio.current_task().cancelled():
+        client_file = os.path.join(os.path.dirname(__file__), "client.http.mts")
+        args = ("tsx", "watch", client_file) if watch else ("tsx", client_file)
+        pid = None
+        try:
+            process = await asyncio.create_subprocess_exec(
+                *args,
+                env={
+                    "LANGGRAPH_HTTP": orjson.dumps(http_config),
+                    "LANGSERVE_GRAPHS": paths_str,
+                    "LANGCHAIN_CALLBACKS_BACKGROUND": "true",
+                    "NODE_ENV": "development" if watch else "production",
+                    "CHOKIDAR_USEPOLLING": "true",
+                    **os.environ,
+                },
+            )
+
+            pid = process.pid
+            logger.info("Started JS HTTP process [%d]", pid)
+
+            code = await process.wait()
+            raise Exception(f"JS HTTP process exited with code {code}")
+
+        except asyncio.CancelledError:
+            logger.info("Shutting down JS HTTP process [%d]", pid or -1)
+            try:
+                process.terminate()
+                await process.wait()
+            except (UnboundLocalError, ProcessLookupError):
+                pass
+            raise
+        except Exception:
+            if attempt >= 3:
+                raise
+            else:
+                logger.warning(f"Retrying JS HTTP process {3 - attempt} more times...")
+                attempt += 1
+
+
 def _get_passthrough_checkpointer(conn: AsyncConnectionProto):
-    from langgraph_storage.checkpoint import Checkpointer
+    from langgraph_runtime.checkpoint import Checkpointer
 
     class PassthroughSerialiser(SerializerProtocol):
         def dumps(self, obj: Any) -> bytes:
@@ -345,7 +455,7 @@ def _get_passthrough_checkpointer(conn: AsyncConnectionProto):
 
 
 def _get_passthrough_store():
-    from langgraph_storage.store import Store
+    from langgraph_runtime.store import Store
 
     return Store()
 
@@ -353,7 +463,7 @@ def _get_passthrough_store():
 # Setup a HTTP server on top of CHECKPOINTER_SOCKET unix socket
 # used by `client.mts` to communicate with the Python checkpointer
 async def run_remote_checkpointer():
-    from langgraph_storage.database import connect
+    from langgraph_runtime.database import connect
 
     async def checkpointer_list(payload: dict):
         """Search checkpoints"""
@@ -363,7 +473,7 @@ async def run_remote_checkpointer():
             checkpointer = _get_passthrough_checkpointer(conn)
             async for item in checkpointer.alist(
                 config=payload.get("config"),
-                limit=payload.get("limit"),
+                limit=int(payload.get("limit") or 10),
                 before=payload.get("before"),
                 filter=payload.get("filter"),
             ):
@@ -438,7 +548,7 @@ async def run_remote_checkpointer():
                     ListNamespacesOp(
                         match_conditions=tuple(op["match_conditions"]),
                         max_depth=op.get("max_depth"),
-                        limit=op.get("limit", 100),
+                        limit=int(op.get("limit", 100)),
                         offset=op.get("offset", 0),
                     )
                 )
@@ -605,60 +715,298 @@ async def run_remote_checkpointer():
     await server.serve()
 
 
+class DisableHttpxLoggingContextManager(AbstractContextManager):
+    """
+    Disable HTTP/1.1 200 OK logs spamming stdout.
+    """
+
+    filter: logging.Filter
+
+    def __init__(self, filter: Callable[[logging.LogRecord], bool] | None = None):
+        self.filter = filter or (lambda record: "200 OK" not in record.getMessage())
+
+    def __enter__(self):
+        logging.getLogger("httpx").addFilter(self.filter)
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        logging.getLogger("httpx").removeFilter(self.filter)
+
+
 async def wait_until_js_ready():
-    async with (
-        httpx.AsyncClient(
-            base_url=f"http://localhost:{GRAPH_PORT}",
-            limits=httpx.Limits(max_connections=1),
-            transport=httpx.AsyncHTTPTransport(verify=SSL),
-        ) as graph_client,
-        httpx.AsyncClient(
-            base_url=f"http://localhost:{REMOTE_PORT}",
-            limits=httpx.Limits(max_connections=1),
-            transport=httpx.AsyncHTTPTransport(verify=SSL),
-        ) as checkpointer_client,
-    ):
-        attempt = 0
-        while not asyncio.current_task().cancelled():
+    with DisableHttpxLoggingContextManager():
+        async with (
+            httpx.AsyncClient(
+                base_url=f"http://localhost:{GRAPH_PORT}",
+                limits=httpx.Limits(max_connections=1),
+                transport=httpx.AsyncHTTPTransport(verify=SSL),
+            ) as graph_client,
+            httpx.AsyncClient(
+                base_url=f"http://localhost:{REMOTE_PORT}",
+                limits=httpx.Limits(max_connections=1),
+                transport=httpx.AsyncHTTPTransport(verify=SSL),
+            ) as checkpointer_client,
+        ):
+            attempt = 0
+            while not asyncio.current_task().cancelled():
+                try:
+                    res = await graph_client.get("/ok")
+                    res.raise_for_status()
+                    res = await checkpointer_client.get("/ok")
+                    res.raise_for_status()
+                    return
+                except httpx.HTTPError:
+                    if attempt > 240:
+                        raise
+                    else:
+                        attempt += 1
+                        await asyncio.sleep(0.5)
+
+
+async def js_healthcheck():
+    with DisableHttpxLoggingContextManager():
+        async with (
+            httpx.AsyncClient(
+                base_url=f"http://localhost:{GRAPH_PORT}",
+                limits=httpx.Limits(max_connections=1),
+                transport=httpx.AsyncHTTPTransport(verify=SSL),
+            ) as graph_client,
+            httpx.AsyncClient(
+                base_url=f"http://localhost:{REMOTE_PORT}",
+                limits=httpx.Limits(max_connections=1),
+                transport=httpx.AsyncHTTPTransport(verify=SSL),
+            ) as checkpointer_client,
+        ):
             try:
                 res = await graph_client.get("/ok")
                 res.raise_for_status()
                 res = await checkpointer_client.get("/ok")
                 res.raise_for_status()
-                return
-            except httpx.HTTPError:
-                if attempt > 240:
-                    raise
-                else:
-                    attempt += 1
-                    await asyncio.sleep(0.5)
+                return True
+            except httpx.HTTPError as exc:
+                logger.warning(
+                    "JS healthcheck failed. Either the JS server is not running or the event loop is blocked by a CPU-intensive task.",
+                    error=exc,
+                )
+                raise HTTPException(
+                    status_code=500,
+                    detail="JS healthcheck failed. Either the JS server is not running or the event loop is blocked by a CPU-intensive task.",
+                ) from exc
 
 
-async def js_healthcheck():
-    async with (
-        httpx.AsyncClient(
-            base_url=f"http://localhost:{GRAPH_PORT}",
-            limits=httpx.Limits(max_connections=1),
+class CustomJsAuthBackend(AuthenticationBackend):
+    ls_auth: AuthenticationBackend | None
+
+    def __init__(self, disable_studio_auth: bool = False):
+        self.ls_auth = None
+        if not disable_studio_auth and LANGGRAPH_AUTH_TYPE == "langsmith":
+            from langgraph_api.auth.langsmith.backend import LangsmithAuthBackend
+
+            self.ls_auth = LangsmithAuthBackend()
+
+    async def authenticate(
+        self, conn: HTTPConnection
+    ) -> tuple[AuthCredentials, BaseUser] | None:
+        if self.ls_auth is not None and (
+            (auth_scheme := conn.headers.get("x-auth-scheme"))
+            and auth_scheme == "langsmith"
+        ):
+            return await self.ls_auth.authenticate(conn)
+
+        headers = dict(conn.headers)
+        # need to remove content-length to prevent confusing the HTTP client
+        headers.pop("content-length", None)
+        headers["x-langgraph-auth-url"] = str(conn.url)
+        headers["x-langgraph-auth-method"] = conn.scope.get("method")
+
+        res = await _client.post("/auth/authenticate", headers=headers)
+        data = res.json()
+
+        if data.get("error"):
+            status = data.get("status") or 403
+            headers = data.get("headers")
+            message = data.get("message") or "Unauthorized"
+
+            raise HTTPException(status_code=status, detail=message, headers=headers)
+
+        return AuthCredentials(data["scopes"]), ProxyUser(DotDict(data["user"]))
+
+
+async def handle_js_auth_event(
+    ctx: Auth.types.AuthContext | None,
+    value: dict,
+) -> Auth.types.FilterType | None:
+    res = await _client.post(
+        "/auth/authorize",
+        headers={"Content-Type": "application/json"},
+        data=json_dumpb(
+            {
+                "resource": ctx.resource,
+                "action": ctx.action,
+                "value": value,
+                "context": {
+                    "user": cast(DotDict, ctx.user).dict(),
+                    "scopes": ctx.permissions,
+                }
+                if ctx
+                else None,
+            }
+        ),
+    )
+
+    response = res.json()
+
+    if response.get("error"):
+        status = response.get("status") or 403
+        headers = response.get("headers")
+        message = response.get("message") or "Unauthorized"
+
+        raise HTTPException(status_code=status, detail=message, headers=headers)
+
+    filters = cast(Auth.types.FilterType | None, response.get("filters"))
+
+    # mutate metadata in value if applicable
+    # we need to preserve the identity of the object, so cannot create a new
+    # dictionary, otherwise the changes will not persist
+    if isinstance(value, dict) and (updated_value := response.get("value")):
+        if isinstance(value.get("metadata"), dict) and (
+            metadata := updated_value.get("metadata")
+        ):
+            value["metadata"].update(metadata)
+
+    return filters
+
+
+class JSCustomHTTPProxyMiddleware:
+    def __init__(self, app: types.ASGIApp) -> None:
+        self.app = app
+        self.proxy_client = httpx.AsyncClient(
+            base_url=f"http://localhost:{GRAPH_HTTP_PORT}",
+            timeout=httpx.Timeout(None),
+            limits=httpx.Limits(),
             transport=httpx.AsyncHTTPTransport(verify=SSL),
-        ) as graph_client,
-        httpx.AsyncClient(
-            base_url=f"http://localhost:{REMOTE_PORT}",
-            limits=httpx.Limits(max_connections=1),
-            transport=httpx.AsyncHTTPTransport(verify=SSL),
-        ) as checkpointer_client,
-    ):
-        try:
-            res = await graph_client.get("/ok")
-            res.raise_for_status()
-            res = await checkpointer_client.get("/ok")
-            res.raise_for_status()
-            return True
-        except httpx.HTTPError as exc:
-            logger.warning(
-                "JS healthcheck failed. Either the JS server is not running or the event loop is blocked by a CPU-intensive task.",
-                error=exc,
+        )
+
+    async def __call__(
+        self, scope: types.Scope, receive: types.Receive, send: types.Send
+    ) -> None:
+        if scope["type"] != "http" or "__langgraph_check" in scope["path"]:
+            # TODO: add support for proxying `websockets``
+            await self.app(scope, receive, send)
+            return
+
+        # First, check if the request can be handled by the JS server
+        with DisableHttpxLoggingContextManager(
+            filter=lambda record: "__langgraph_check" not in record.getMessage()
+        ):
+            res = await self.proxy_client.options(
+                "/__langgraph_check",
+                headers={
+                    "x-langgraph-method": scope["method"],
+                    "x-langgraph-path": scope["path"],
+                },
             )
-            raise HTTPException(
-                status_code=500,
-                detail="JS healthcheck failed. Either the JS server is not running or the event loop is blocked by a CPU-intensive task.",
-            ) from exc
+
+        input_buffer: deque[types.Message] = deque()
+
+        async def yield_to_python(node_request: httpx.Response):
+            nonlocal input_buffer
+
+            async def replay_request():
+                if input_buffer:
+                    if node_request.headers.get("x-langgraph-body") == "true":
+                        input_buffer.clear()
+                        return {
+                            "type": "http.request",
+                            "body": await node_request.aread(),
+                            "more_body": False,
+                        }
+
+                    return input_buffer.popleft()
+                else:
+                    return await receive()
+
+            async def send_with_extra_headers(message: types.Message):
+                if message["type"] == "http.response.start":
+                    headers = MutableHeaders(scope=message)
+                    for k, v in node_request.headers.items():
+                        if k in (
+                            "content-length",
+                            "content-encoding",
+                            "content-type",
+                            "transfer-encoding",
+                            "connection",
+                            "keep-alive",
+                            "x-langgraph-body",
+                            "x-langgraph-status",
+                        ):
+                            continue
+
+                        # Respect existing headers set by the Python server
+                        headers.append(k, headers.get(k, None) or v)
+
+                await send(message)
+
+            return await self.app(scope, replay_request, send_with_extra_headers)
+
+        # If the JS server does not handle the request, yield the control back to the
+        # Python server.
+        if not res.is_success:
+            return await yield_to_python(res)
+
+        # Stream request body
+        async def upload_request_body() -> AsyncIterator[bytes]:
+            nonlocal input_buffer
+
+            more_body = True
+            while more_body:
+                message = await receive()
+                input_buffer.append(message)
+
+                more_body = message.get("more_body", False)
+                yield message.get("body", b"")
+
+        # Make the proxied request
+        async with self.proxy_client.stream(
+            scope["method"],
+            scope["path"],
+            params=scope["query_string"],
+            headers={
+                k.decode("latin-1"): v.decode("latin-1")
+                for k, v in scope["headers"]
+                if k.lower() not in (b"host", b"content-length")
+            },
+            content=upload_request_body(),
+        ) as response:
+            if (
+                response.status_code == 404
+                and response.headers.get("x-langgraph-status") == "not-found"
+            ):
+                return await yield_to_python(response)
+
+            # Send the response headers
+            await send(
+                {
+                    "type": "http.response.start",
+                    "status": response.status_code,
+                    "headers": [
+                        (k.encode("latin-1"), v.encode("latin-1"))
+                        for k, v in response.headers.items()
+                        if k.lower() not in (b"transfer-encoding",)
+                    ],
+                }
+            )
+
+            # Stream the response body
+            async for chunk in response.aiter_raw():
+                await send(
+                    {"type": "http.response.body", "body": chunk, "more_body": True}
+                )
+
+            # Send the final empty chunk to indicate the end of the response
+            await send({"type": "http.response.body", "body": b"", "more_body": False})
+
+    async def __aenter__(self):
+        return self
+
+    async def __aexit__(self, exc_type, exc_val, exc_tb):
+        await self.proxy_client.aclose()
