@@ -6,9 +6,6 @@ import structlog
 from starlette.config import Config
 from structlog.typing import EventDict
 
-from langgraph_api.metadata import append_log
-from langgraph_api.serde import json_dumpb
-
 # env
 
 log_env = Config()
@@ -52,6 +49,8 @@ class JSONRenderer:
         """
         The return type of this depends on the return type of self._dumps.
         """
+        from langgraph_api.serde import json_dumpb
+
         return json_dumpb(event_dict).decode()
 
 
@@ -69,6 +68,8 @@ class TapForMetadata:
             event_dict["logger"].startswith("langgraph")
             and LEVELS[event_dict["level"].upper()] > LEVELS["INFO"]
         ):
+            from langgraph_api.metadata import append_log
+
             append_log(event_dict.copy())
         return event_dict
 
@@ -118,13 +119,14 @@ class Formatter(structlog.stdlib.ProcessorFormatter):
 
 # configure structlog
 
-structlog.configure(
-    processors=[
-        structlog.stdlib.filter_by_level,
-        *shared_processors,
-        structlog.stdlib.ProcessorFormatter.wrap_for_formatter,
-    ],
-    logger_factory=structlog.stdlib.LoggerFactory(),
-    wrapper_class=structlog.stdlib.BoundLogger,
-    cache_logger_on_first_use=True,
-)
+if not structlog.is_configured():
+    structlog.configure(
+        processors=[
+            structlog.stdlib.filter_by_level,
+            *shared_processors,
+            structlog.stdlib.ProcessorFormatter.wrap_for_formatter,
+        ],
+        logger_factory=structlog.stdlib.LoggerFactory(),
+        wrapper_class=structlog.stdlib.BoundLogger,
+        cache_logger_on_first_use=True,
+    )
