@@ -1,9 +1,11 @@
 import asyncio
+import urllib.parse
 import uuid
 from collections.abc import Mapping, Sequence
 from typing import Any, NamedTuple, TypedDict
 from uuid import UUID
 
+import orjson
 from langgraph.checkpoint.base.id import uuid6
 from starlette.authentication import BaseUser
 from starlette.exceptions import HTTPException
@@ -21,7 +23,7 @@ from langgraph_api.schema import (
     StreamMode,
 )
 from langgraph_api.utils import AsyncConnectionProto, get_auth_ctx
-from langgraph_storage.ops import Runs, logger
+from langgraph_runtime.ops import Runs, logger
 
 
 class RunCreateDict(TypedDict):
@@ -152,6 +154,11 @@ def get_user_id(user: BaseUser | None) -> str | None:
             pass
 
 
+LANGSMITH_METADATA = "langsmith-metadata"
+LANGSMITH_TAGS = "langsmith-tags"
+LANGSMITH_PROJECT = "langsmith-project"
+
+
 async def create_valid_run(
     conn: AsyncConnectionProto,
     thread_id: str | None,
@@ -170,18 +177,26 @@ async def create_valid_run(
         payload,
         run_id=run_id,
     )
+    if (
+        thread_id is None
+        and (command := payload.get("command"))
+        and command.get("resume")
+    ):
+        raise HTTPException(
+            status_code=400,
+            detail="You must provide a thread_id when resuming.",
+        )
     temporary = thread_id is None and payload.get("on_completion", "delete") == "delete"
     stream_mode, multitask_strategy, prevent_insert_if_inflight = assign_defaults(
         payload
     )
-
     # assign custom headers and checkpoint to config
     config = payload.get("config") or {}
-    config.setdefault("configurable", {})
+    configurable = config.setdefault("configurable", {})
     if checkpoint_id:
-        config["configurable"]["checkpoint_id"] = str(checkpoint_id)
+        configurable["checkpoint_id"] = str(checkpoint_id)
     if checkpoint := payload.get("checkpoint"):
-        config["configurable"].update(checkpoint)
+        configurable.update(checkpoint)
     for key, value in headers.items():
         if key.startswith("x-"):
             if key in (
@@ -190,14 +205,27 @@ async def create_valid_run(
                 "x-service-key",
             ):
                 continue
-            config["configurable"][key] = value
+            configurable[key] = value
+        elif key == "langsmith-trace":
+            configurable[key] = value
+            if baggage := headers.get("baggage"):
+                for item in baggage.split(","):
+                    key, value = item.split("=")
+                    if key == LANGSMITH_METADATA and key not in configurable:
+                        configurable[key] = orjson.loads(urllib.parse.unquote(value))
+                    elif key == LANGSMITH_TAGS:
+                        configurable[key] = urllib.parse.unquote(value).split(",")
+                    elif key == LANGSMITH_PROJECT:
+                        configurable[key] = urllib.parse.unquote(value)
+        elif key == "user-agent":
+            configurable[key] = value
     ctx = get_auth_ctx()
     if ctx:
         user = ctx.user
         user_id = get_user_id(user)
-        config["configurable"]["langgraph_auth_user"] = user
-        config["configurable"]["langgraph_auth_user_id"] = user_id
-        config["configurable"]["langgraph_auth_permissions"] = ctx.permissions
+        configurable["langgraph_auth_user"] = user
+        configurable["langgraph_auth_user_id"] = user_id
+        configurable["langgraph_auth_permissions"] = ctx.permissions
     else:
         user_id = None
     run_coro = Runs.put(
@@ -241,7 +269,17 @@ async def create_valid_run(
     # handle multitask strategy
     inflight_runs = [run async for run in run_]
     if first["run_id"] == run_id:
-        logger.info("Created run", run_id=str(run_id), thread_id=str(thread_id))
+        logger.info(
+            "Created run",
+            run_id=str(run_id),
+            thread_id=str(thread_id),
+            assistant_id=str(assistant_id),
+            multitask_strategy=multitask_strategy,
+            stream_mode=stream_mode,
+            temporary=temporary,
+            after_seconds=payload.get("after_seconds", 0),
+            if_not_exists=payload.get("if_not_exists", "reject"),
+        )
         # inserted, proceed
         if multitask_strategy in ("interrupt", "rollback") and inflight_runs:
             try:
