@@ -19,20 +19,20 @@ from langgraph.errors import (
     InvalidUpdateError,
 )
 from langgraph.pregel.debug import CheckpointPayload, TaskResultPayload
-from langgraph.types import Command, Send
 from pydantic import ValidationError
 from pydantic.v1 import ValidationError as ValidationErrorLegacy
 
 from langgraph_api.asyncio import ValueEvent, wait_if_not_done
+from langgraph_api.command import map_cmd
 from langgraph_api.graph import get_graph
 from langgraph_api.js.base import BaseRemotePregel
-from langgraph_api.metadata import HOST, PLAN, incr_nodes
-from langgraph_api.schema import Run, RunCommand, StreamMode
+from langgraph_api.metadata import HOST, PLAN, USER_API_URL, incr_nodes
+from langgraph_api.schema import Run, StreamMode
 from langgraph_api.serde import json_dumpb
 from langgraph_api.utils import AsyncConnectionProto
-from langgraph_storage.checkpoint import Checkpointer
-from langgraph_storage.ops import Runs
-from langgraph_storage.store import Store
+from langgraph_runtime.checkpoint import Checkpointer
+from langgraph_runtime.ops import Runs
+from langgraph_runtime.store import Store
 
 logger = structlog.stdlib.get_logger(__name__)
 
@@ -70,32 +70,6 @@ def _preprocess_debug_checkpoint(payload: CheckpointPayload | None) -> dict[str,
     return payload
 
 
-def _map_cmd(cmd: RunCommand) -> Command:
-    goto = cmd.get("goto")
-    if goto is not None and not isinstance(goto, list):
-        goto = [cmd.get("goto")]
-
-    update = cmd.get("update")
-    if isinstance(update, tuple | list) and all(
-        isinstance(t, tuple | list) and len(t) == 2 and isinstance(t[0], str)
-        for t in update
-    ):
-        update = [tuple(t) for t in update]
-
-    return Command(
-        update=update,
-        goto=(
-            [
-                it if isinstance(it, str) else Send(it["node"], it["input"])
-                for it in goto
-            ]
-            if goto
-            else None
-        ),
-        resume=cmd.get("resume"),
-    )
-
-
 async def astream_state(
     stack: AsyncExitStack,
     conn: AsyncConnectionProto,
@@ -125,7 +99,7 @@ async def astream_state(
     )
     input = kwargs.pop("input")
     if cmd := kwargs.pop("command"):
-        input = _map_cmd(cmd)
+        input = map_cmd(cmd)
     stream_mode: list[StreamMode] = kwargs.pop("stream_mode")
     feedback_keys = kwargs.pop("feedback_keys", None)
     stream_modes_set: set[StreamMode] = set(stream_mode) - {"events"}
@@ -140,6 +114,7 @@ async def astream_state(
     config["metadata"]["langgraph_version"] = langgraph.version.__version__
     config["metadata"]["langgraph_plan"] = PLAN
     config["metadata"]["langgraph_host"] = HOST
+    config["metadata"]["langgraph_api_url"] = USER_API_URL
     # attach node counter
     if not isinstance(graph, BaseRemotePregel):
         config["configurable"]["__pregel_node_finished"] = incr_nodes
@@ -285,12 +260,13 @@ async def consume(stream: AnyStream, run_id: str) -> None:
                     run_id, mode, await run_in_executor(None, json_dumpb, payload)
                 )
         except Exception as e:
+            g = e
             if isinstance(e, ExceptionGroup):
                 e = e.exceptions[0]
             await Runs.Stream.publish(
                 run_id, "error", await run_in_executor(None, json_dumpb, e)
             )
-            raise e from None
+            raise e from g
 
 
 def get_feedback_urls(run_id: str, feedback_keys: list[str]) -> dict[str, str]:
