@@ -10,6 +10,7 @@ from starlette.exceptions import HTTPException
 
 from langgraph_api.auth.custom import SimpleUser, normalize_user
 from langgraph_api.config import (
+    BG_JOB_ISOLATED_LOOPS,
     BG_JOB_MAX_RETRIES,
     BG_JOB_TIMEOUT_SECS,
 )
@@ -25,9 +26,9 @@ from langgraph_api.stream import (
     consume,
 )
 from langgraph_api.utils import set_auth_ctx, with_user
-from langgraph_storage.database import connect
-from langgraph_storage.ops import Runs, Threads
-from langgraph_storage.retry import RETRIABLE_EXCEPTIONS
+from langgraph_runtime.database import connect
+from langgraph_runtime.ops import Runs, Threads
+from langgraph_runtime.retry import RETRIABLE_EXCEPTIONS
 
 try:
     from psycopg.errors import InFailedSqlTransaction
@@ -110,7 +111,22 @@ async def worker(
 
         try:
             if attempt > BG_JOB_MAX_RETRIES:
-                raise RuntimeError(f"Run {run['run_id']} exceeded max attempts")
+                await logger.aerror("Run exceeded max attempts", run_id=run["run_id"])
+
+                error_message = (
+                    f"Run {run['run_id']} exceeded max attempts ({BG_JOB_MAX_RETRIES}).\n\n"
+                    "This may happen if your code blocks the event loop with synchronous I/O bound calls (network requests, database queries, etc.).\n\n"
+                    "If that is the case, your issues may be resolved by converting synchronous operations to async (e.g., use aiohttp instead of requests).\n\n"
+                )
+
+                if not BG_JOB_ISOLATED_LOOPS:
+                    error_message += (
+                        "Also consider setting BG_JOB_ISOLATED_LOOPS=true in your environment. This will isolate I/O-bound operations to avoid"
+                        " blocking the main API server.\n\n"
+                        "See: https://langchain-ai.github.io/langgraph/cloud/reference/env_var/#bg_job_isolated_loops\n\n"
+                    )
+
+                raise RuntimeError(error_message)
             if temporary:
                 stream = astream_state(
                     AsyncExitStack(), conn, cast(Run, run), attempt, done
@@ -150,6 +166,7 @@ async def worker(
                 run_started_at=run_started_at.isoformat(),
                 run_ended_at=run_ended_at,
                 run_exec_ms=ms(datetime.now(UTC), run_started_at),
+                graph_id=_get_graph_id(run),
             )
             await Runs.set_status(conn, run_id, "timeout")
         except UserRollback as e:
@@ -166,6 +183,7 @@ async def worker(
                     run_started_at=run_started_at.isoformat(),
                     run_ended_at=run_ended_at,
                     run_exec_ms=ms(datetime.now(UTC), run_started_at),
+                    graph_id=_get_graph_id(run),
                 )
 
             except InFailedSqlTransaction as e:
@@ -203,6 +221,7 @@ async def worker(
                 run_started_at=run_started_at.isoformat(),
                 run_ended_at=run_ended_at,
                 run_exec_ms=ms(datetime.now(UTC), run_started_at),
+                graph_id=_get_graph_id(run),
             )
             await Runs.set_status(conn, run_id, "interrupted")
         except RETRIABLE_EXCEPTIONS as e:
@@ -210,7 +229,7 @@ async def worker(
             status = "retry"
             run_ended_at = datetime.now(UTC).isoformat()
             await logger.awarning(
-                "Background run failed, will retry",
+                f"Background run failed, will retry. Exception: {e}",
                 exc_info=True,
                 run_id=str(run_id),
                 run_attempt=attempt,
@@ -218,6 +237,7 @@ async def worker(
                 run_started_at=run_started_at.isoformat(),
                 run_ended_at=run_ended_at,
                 run_exec_ms=ms(datetime.now(UTC), run_started_at),
+                graph_id=_get_graph_id(run),
             )
             await Runs.set_status(conn, run_id, "pending")
             raise
@@ -226,7 +246,7 @@ async def worker(
             status = "error"
             run_ended_at = datetime.now(UTC).isoformat()
             await logger.aexception(
-                "Background run failed",
+                f"Background run failed. Exception: {exc}",
                 exc_info=not isinstance(exc, RemoteException),
                 run_id=str(run_id),
                 run_attempt=attempt,
@@ -234,6 +254,7 @@ async def worker(
                 run_started_at=run_started_at.isoformat(),
                 run_ended_at=run_ended_at,
                 run_exec_ms=ms(datetime.now(UTC), run_started_at),
+                graph_id=_get_graph_id(run),
             )
             await Runs.set_status(conn, run_id, "error")
         set_auth_ctx(None, None)
@@ -253,6 +274,7 @@ async def worker(
         # Note we don't handle asyncio.CancelledError here, as we want to
         # let it bubble up and rollback db transaction, thus marking the run
         # as available to be picked up by another worker
+
     return {
         "checkpoint": checkpoint,
         "status": status,
@@ -266,3 +288,11 @@ async def worker(
 
 def ms(after: datetime, before: datetime) -> int:
     return int((after - before).total_seconds() * 1000)
+
+
+def _get_graph_id(run: Run) -> str | None:
+    try:
+        return run["kwargs"]["config"]["configurable"]["graph_id"]
+    except Exception:
+        logger.info(f"Failed to get graph_id from run {run['run_id']}")
+        return "Unknown"
